Package: GenoTriplo
Type: Package
Title: Genotyping Triploids/Tetraploids (or Diploids) from Luminescence
        Data
Version: 2.2.1
Authors@R: 
  c(person(given = "Julien",
           family = "Roche",
           role = c("aut", "cre"),
           email = "julien.roche@inrae.fr"),
    person(given = "Florence",
           family = "Phocas",
           role = "aut"),
    person(given = "Mathieu",
           family = "Besson",
           role = "aut"),
    person(given = "Pierre",
           family = "Patrice",
           role = "aut"),
    person(given = "Marc",
           family = "Vandeputte",
           role = "aut"),
    person(given = "François",
           family = "Allal",
           role = "aut"),
    person(given = "Pierrick",
           family = "Haffray",
           role = "aut"))
Description: Genotyping of triploid individuals from luminescence data (marker probeset A and B). Works also for diploids and tetraploids.
	Three main functions: Create_Dataset_from_file() to build dataset ; Clustering_parallele_from_dir() that regroups individuals with a same genotype based on proximity and
	Genotyping_parallele_from_dir() that assigns a genotype to each cluster. For Shiny interface use: launch_GenoShiny().
License: GPL
Encoding: UTF-8
Imports: cowplot, doParallel, dplyr, DT, foreach, ggplot2, htmltools,
        parallel, processx, rlang, Rmixmod, shiny, shinythemes,
        data.table, shinyFiles, bslib, stringr
Depends: R (>= 3.5.0), shinyBS
NeedsCompilation: no
Packaged: 2026-07-28 14:11:27 UTC; Proprietaire
Author: Julien Roche [aut, cre],
  Florence Phocas [aut],
  Mathieu Besson [aut],
  Pierre Patrice [aut],
  Marc Vandeputte [aut],
  François Allal [aut],
  Pierrick Haffray [aut]
Maintainer: Julien Roche <julien.roche@inrae.fr>
Config/roxygen2/version: 8.0.0
Repository: CRAN
Date/Publication: 2026-07-28 14:50:02 UTC
