# Generated by roxygen2: do not edit by hand

export(Clustering)
export(Create_Dataset)
export(Run_Clustering)
export(Run_Genotyping)
export(launch_GenoShiny)
import(Rmixmod)
import(cowplot)
import(dplyr)
import(foreach)
import(ggplot2)
import(htmltools)
import(processx)
import(shiny, except=c(dataTableOutput, renderDataTable))
import(shinyBS)
import(shinythemes)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(doParallel,registerDoParallel)
importFrom(doParallel,stopImplicitCluster)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(rlang,.data)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(utils,head)
importFrom(utils,memory.limit)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.table)
